"""Example script for a reactive map (zoomable)."""

import json
import os
from geovizpy import Geoviz

# Define output directory
output_dir = os.path.join(os.path.dirname(__file__), "html")
os.makedirs(output_dir, exist_ok=True)

data_path = os.path.join(os.path.dirname(__file__), "data", "world.json")
try:
    with open(data_path) as f:
        world_data = json.load(f)
except FileNotFoundError:
    print(f"{data_path} not found.")
    world_data = {}

viz = Geoviz(projection="polar", width=700, zoomable=True)
viz.outline()
viz.graticule(stroke="white", strokeWidth=0.4)
viz.path(datum=world_data, fill="white", fillOpacity=0.3)

viz.prop(
    id="bubbles",
    symbol="circle",
    data=world_data,
    var="pop",
    fill="#F13C47",
    dodge=False,
    k=50,
    leg_title="Population",
    tip=True
)

print("Generating a static snapshot of the map.")
viz.render_html(os.path.join(output_dir, "reactive_snapshot.html"))



"""Module for interactive map controls."""

class ControlsMixin:
    """Mixin class for interactive map controls."""

    def add_layer_control(self, layers=None, pos=None, x=10, y=10, title="Layers"):
        """
        Add a collapsible layer control widget (expands on hover).
        
        Args:
            layers (list): List of layer IDs to control. If None, finds all layers with IDs.
            pos (string): Predefined position ("top-right", "top-left", etc.). Overrides x, y.
            x (int): X position of the control.
            y (int): Y position of the control.
            title (string): Title of the control panel.
        """
        self.layer_control_config = {"layers": layers, "pos": pos, "x": x, "y": y, "title": title}
        return self

    def add_export_control(self, pos=None, x=10, y=50, title="Export"):
        """
        Add a download button to export the map as SVG or PNG (expands on hover).
        
        Args:
            pos (string): Predefined position ("top-right", "top-left", etc.). Overrides x, y.
            x (int): X position of the control.
            y (int): Y position of the control.
            title (string): Title of the button.
        """
        self.export_control_config = {"pos": pos, "x": x, "y": y, "title": title}
        return self

    def add_opacity_control(self, layers, pos=None, x=10, y=90, title="Opacity"):
        """
        Add a slider to control the opacity of one or multiple layers.
        
        Args:
            layers (string or list): The ID(s) of the layer(s) to control.
            pos (string): Predefined position ("top-right", "top-left", etc.). Overrides x, y.
            x (int): X position of the control.
            y (int): Y position of the control.
            title (string): Title of the control.
        """
        if not hasattr(self, "opacity_control_configs"):
            self.opacity_control_configs = []
        
        if isinstance(layers, str):
            layers = [layers]
            
        self.opacity_control_configs.append({"layers": layers, "pos": pos, "x": x, "y": y, "title": title})
        return self



"""Example script for exporting the map configuration to JSON."""

import json
import os
from geovizpy import Geoviz

# Define output directory
output_dir = os.path.join(os.path.dirname(__file__), "html")
os.makedirs(output_dir, exist_ok=True)

data_path = os.path.join(os.path.dirname(__file__), "data", "world.json")
try:
    with open(data_path) as f:
        world_data = json.load(f)
except FileNotFoundError:
    print(f"{data_path} not found.")
    world_data = {}

viz = Geoviz(projection="EqualEarth")
viz.outline()
viz.graticule(stroke="white", strokeWidth=0.4)
viz.choro(
    data=world_data,
    var="gdppc",
    strokeWidth=0.3,
    tip=True,
    leg_title="GDP per capita"
)
viz.header(text="Map from JSON Config", fontSize=30)

config_json = viz.to_json()
print("Configuration JSON (first 500 chars):")
print(config_json[:500] + "...")

with open(os.path.join(output_dir, "map_config.json"), "w") as f:
    f.write(config_json)
print(f"\nConfiguration saved to {os.path.join(output_dir, 'map_config.json')}")



"""Example script for creating a proportional symbol map (bubble map)."""

import json
import os
from geovizpy import Geoviz

# Define output directory
output_dir = os.path.join(os.path.dirname(__file__), "html")
os.makedirs(output_dir, exist_ok=True)

data_path = os.path.join(os.path.dirname(__file__), "data", "world.json")
try:
    with open(data_path) as f:
        world_data = json.load(f)
except FileNotFoundError:
    print(f"{data_path} not found.")
    world_data = {}

viz = Geoviz(projection="EqualEarth", zoomable=True)
viz.outline()
viz.graticule(stroke="white", strokeWidth=0.4)
viz.path(datum=world_data, fill="white", fillOpacity=0.4)

# Use viz.prop instead of viz.circle to get automatic legend handling
viz.prop(
    data=world_data,
    var="pop", # 'var' is used instead of 'r' in viz.prop
    fill="#f07d75",
    tip="(d) => `${d.properties.NAMEen}\n${d.properties.pop / 1000} thousands inh.`",
    leg_type="separate",
    leg_title="Population",
    leg_pos=[30, 30], # Added position to make sure it's visible
    leg_title_fontSize=15
)

pops = [f["properties"]["pop"] for f in world_data.get("features", []) if "pop" in f["properties"]]

viz.header(
    fontSize=30,
    text="A World Map With Bubbles",
    fill="#267A8A",
    fontWeight="bold",
    fontFamily="Tangerine"
)

viz.render_html(os.path.join(output_dir, "bubble.html"))



"""Example script for layer control and export functionality."""

import json
import os
from geovizpy import Geoviz

# Define output directory
output_dir = os.path.join(os.path.dirname(__file__), "html")
os.makedirs(output_dir, exist_ok=True)

# Load world data
data_path = os.path.join(os.path.dirname(__file__), "data", "world.json")
try:
    with open(data_path) as f:
        world_data = json.load(f)
except FileNotFoundError:
    print(f"{data_path} not found.")
    world_data = {}

# Create a map
viz = Geoviz(projection="EqualEarth", zoomable=True)
viz.outline(id="outline")
viz.graticule(stroke="white", strokeWidth=0.4, id="graticule")

# Add a choropleth layer with an ID and legend position
viz.choro(
    data=world_data,
    var="gdppc",
    colors="Reds",
    id="choropleth_gdp",
    legend=True,
    leg_title="GDP per Capita",
    leg_pos=[10, 100]  # Position for the choro legend
)

# Add a proportional symbol layer with an ID and legend position
viz.prop(
    data=world_data,
    var="pop",
    fill="#4f8a8b",
    stroke="white",
    id="prop_population",
    legend=True,
    leg_title="Population",
    leg_pos=[10, 250]  # Position for the prop legend
)

# Add the layer control widget (hover to expand)
viz.add_layer_control(
    layers=["choropleth_gdp", "prop_population", "graticule"],
    title="Layers",
    x=10,
    y=10
)

# Add the export control widget (hover to expand)
viz.add_export_control(
    pos="top-right" # Or use x, y
)

# Add opacity control for multiple layers
viz.add_opacity_control(
    layers=["choropleth_gdp", "prop_population"],
    title="Layers Opacity",
    x=10,
    y=90
)

viz.header(text="Map with Layer & Export Controls", fontSize=20)
viz.render_html(os.path.join(output_dir, "layer_control_map.html"))



"""Example script demonstrating advanced plot types."""

import json
import os
import random
from geovizpy import Geoviz

# Define output directory
output_dir = os.path.join(os.path.dirname(__file__), "html")
os.makedirs(output_dir, exist_ok=True)

data_path = os.path.join(os.path.dirname(__file__), "data", "world.json")
try:
    with open(data_path) as f:
        world_data = json.load(f)
except FileNotFoundError:
    print(f"{data_path} not found.")
    world_data = {}

continents = ["Africa", "Asia", "Europe", "North America", "South America", "Oceania"]
symbols_list = ["star", "circle", "cross", "diamond", "square", "triangle"]

for f in world_data.get("features", []):
    f["properties"]["random_continent"] = random.choice(continents)
    f["properties"]["random_symbol"] = random.choice(symbols_list)

# 1. Typology Map
viz1 = Geoviz(projection="EqualEarth", width=800)
viz1.outline()
viz1.graticule(stroke="white", strokeWidth=0.4)

viz1.typo(
    data=world_data,
    var="random_continent",
    colors="Pastel",
    strokeWidth=0.5,
    leg_title="Random Continent",
    leg_pos=[10, 200]
)
viz1.header(text="Typology Map (Random Data)", fontSize=20)
viz1.render_html(os.path.join(output_dir, "advanced_typo.html"))

# 2. Proportional Symbols with Typology
viz2 = Geoviz(projection="EqualEarth", width=800)
viz2.outline()
viz2.graticule()
viz2.path(datum=world_data, fill="#eee")

viz2.proptypo(
    data=world_data,
    var1="pop",
    var2="random_continent",
    k=50,
    colors="Set1",
    leg1_title="Population",
    leg2_title="Continent",
    leg1_pos=[10, 300],
    leg2_pos=[10, 150]
)
viz2.header(text="Proportional Symbols + Typology", fontSize=20)
viz2.render_html(os.path.join(output_dir, "advanced_proptypo.html"))

# 3. Symbol Map
viz3 = Geoviz(projection="EqualEarth", width=800)
viz3.outline()
viz3.graticule()
viz3.path(datum=world_data, fill="#ddd")

viz3.picto(
    data=world_data,
    var="random_symbol",
    symbols=["star", "circle", "cross", "diamond", "square", "triangle"],
    colors="black",
    fill="red",
    stroke="white",
    strokeWidth=1,
    k=15,
    leg_title="Random Symbols",
    leg_pos=[10, 200]
)
viz3.header(text="Symbol Map (Picto)", fontSize=20)
viz3.render_html(os.path.join(output_dir, "advanced_symbol.html"))



"""Module for rendering the map to HTML and JSON."""

import json
import tempfile
import os
import time
import html
import sys
import subprocess

class RendererMixin:
    """Mixin class for rendering the map."""

    def get_config(self):
        """Return the configuration as a JSON-compatible list of commands."""
        def process_args(args):
            new_args = {}
            for k, v in args.items():
                if v is None:
                    continue
                if isinstance(v, str) and (v.strip().startswith("(") or v.strip().startswith("function") or "=>" in v or v.strip().startswith("d3.")):
                     new_args[k] = {"__js_func__": v}
                elif isinstance(v, dict):
                    new_args[k] = process_args(v)
                else:
                    new_args[k] = v
            return new_args

        processed_commands = []
        for cmd in self.commands:
            processed_commands.append({"name": cmd["name"], "args": process_args(cmd["args"])})
        
        return processed_commands

    def to_json(self):
        """Return the configuration as a JSON string."""
        return json.dumps(self.get_config())

    def _get_html_content(self):
        """Generate the full HTML content string."""
        json_commands = self.to_json()
        
        layer_control_js = self._get_layer_control_js()
        export_control_js = self._get_export_control_js()
        opacity_control_js = self._get_opacity_control_js()

        return f"""
<!DOCTYPE html>
<html>
<head>
  <meta charset="UTF-8" />
  <link rel="stylesheet" href="https://fonts.googleapis.com/css?family=Tangerine"/>
  <script src="https://cdn.jsdelivr.net/npm/d3@7"></script>
  <script src="https://cdn.jsdelivr.net/npm/geoviz@0.9.8"></script>
  <style>
    body {{ margin: 0; padding: 0; overflow: hidden; }}
    button {{ background: #f8f9fa; border: 1px solid #ddd; border-radius: 3px; }}
    button:hover {{ background: #e2e6ea; }}
  </style>
</head>
<body>
<div id="geoviz-container" style="position: relative; display: inline-block;"></div>
<script>
  const commands = {json_commands};
  let svg;

  // Helper to revive functions
  function revive(obj) {{
    if (typeof obj === 'object' && obj !== null) {{
      if (obj.hasOwnProperty('__js_func__')) {{
         try {{
            return eval(obj['__js_func__']);
         }} catch (e) {{
            console.error("Failed to eval function:", obj['__js_func__'], e);
            return null;
         }}
      }} else {{
         for (let key in obj) {{
            obj[key] = revive(obj[key]);
         }}
      }}
    }}
    return obj;
  }}

  const revivedCommands = revive(commands);

  revivedCommands.forEach(cmd => {{
    if (cmd.name === "create") {{
      svg = geoviz.create(cmd.args);
    }} else {{
      const parts = cmd.name.split(".");
      if (parts.length === 1) {{
         if (svg[parts[0]]) {{
            svg[parts[0]](cmd.args);
         }} else {{
            console.warn("Method " + parts[0] + " not found");
         }}
      }} else if (parts.length === 2) {{
         if (svg[parts[0]] && svg[parts[0]][parts[1]]) {{
            svg[parts[0]][parts[1]](cmd.args);
         }} else {{
            console.warn("Method " + cmd.name + " not found");
         }}
      }}
    }}
  }});

  if (svg) {{
    document.getElementById("geoviz-container").appendChild(svg.render());
  }}
  
  {layer_control_js}
  {export_control_js}
  {opacity_control_js}
</script>
</body>
</html>
"""

    def render_html(self, filename="map.html"):
        """Render the map to an HTML file."""
        html_content = self._get_html_content()
        with open(filename, "w") as f:
            f.write(html_content)
        print(f"Map saved to {filename}")

    def show(self, width=800, height=600):
        """
        Display the map in a Jupyter notebook using an IFrame.
        
        Args:
            width (int/str): Width of the display area (default 800).
            height (int/str): Height of the display area (default 600).
        """
        try:
            from IPython.display import IFrame
            import base64
        except ImportError:
            print("IPython is required to display the map. Please install it with 'pip install ipython'.")
            return

        html_content = self._get_html_content()
        b64_content = base64.b64encode(html_content.encode('utf-8')).decode('utf-8')
        data_uri = f"data:text/html;base64,{b64_content}"
        
        return IFrame(src=data_uri, width=width, height=height)

    def save(self, filename="map.html"):
        """
        Save the map to a file.
        
        If filename ends with .html, saves the interactive map.
        If filename ends with .png or .svg, saves a static image.
        
        For image export, 'playwright' is required. Install it with:
        pip install geovizpy[export]
        playwright install
        """
        if filename.endswith(".html"):
            self.render_html(filename)
        elif filename.endswith(".png") or filename.endswith(".svg"):
            self._save_image(filename)
        else:
            print("Error: filename must end with .html, .png, or .svg")

    def _save_image(self, filename):
        """Internal method to save as PNG or SVG using Playwright via a subprocess."""
        
        # Check if playwright is installed
        try:
            import playwright
        except ImportError:
            print("Error: Playwright is required for image export.")
            print("Please install it with: pip install geovizpy[export] && playwright install")
            return

        # Create a temporary HTML file
        with tempfile.NamedTemporaryFile(mode="w", delete=False, suffix=".html") as tmp_file:
            self.render_html(tmp_file.name)
            tmp_path = tmp_file.name

        # Create a temporary Python script to run Playwright
        # This isolates Playwright from the current asyncio loop (Jupyter)
        script_content = f"""
import os
from playwright.sync_api import sync_playwright

def run():
    try:
        with sync_playwright() as p:
            browser = p.chromium.launch()
            page = browser.new_page(viewport={{"width": 1000, "height": 800}})
            page.goto(f"file://{{os.path.abspath('{tmp_path}')}}")
            page.wait_for_timeout(2000)

            if "{filename}".endswith(".svg"):
                svg_outer = page.locator("svg").first.evaluate("el => el.outerHTML")
                with open("{filename}", "w") as f:
                    f.write(svg_outer)
            else:
                page.locator("svg").first.screenshot(path="{filename}")
            
            browser.close()
            print(f"Image saved to {filename}")
    except Exception as e:
        print(f"Error in subprocess: {{e}}")
        exit(1)

if __name__ == "__main__":
    run()
"""
        
        with tempfile.NamedTemporaryFile(mode="w", delete=False, suffix=".py") as tmp_script:
            tmp_script.write(script_content)
            tmp_script_path = tmp_script.name

        try:
            # Run the script in a subprocess
            result = subprocess.run([sys.executable, tmp_script_path], capture_output=True, text=True)
            if result.returncode != 0:
                print(f"Error saving image: {result.stderr}")
            else:
                print(result.stdout.strip())
        finally:
            # Cleanup
            if os.path.exists(tmp_path):
                os.remove(tmp_path)
            if os.path.exists(tmp_script_path):
                os.remove(tmp_script_path)

    def _get_layer_control_js(self):
        if not self.layer_control_config:
            return ""
        config = self.layer_control_config
        layers_json = json.dumps(config["layers"]) if config["layers"] else "null"
        return f"""
            const layerConfig = {{
                layers: {layers_json},
                pos: "{config.get('pos')}",
                x: {config.get('x', 10)},
                y: {config.get('y', 10)},
                title: "{config.get('title', 'Layers')}"
            }};
            
            function createLayerControl() {{
                const wrapper = document.createElement("div");
                wrapper.style.position = "absolute";
                wrapper.style.zIndex = "1000";
                
                const button = document.createElement("div");
                button.innerHTML = `<svg width="24" height="24" viewBox="0 0 24 24" fill="#333"><path d="M11.99 18.54l-7.37-5.73L3 14.07l9 7 9-7-1.63-1.27-7.38 5.74zM12 16l7.36-5.73L21 9l-9-7-9 7 1.63 1.27L12 16z"/></svg>`;
                button.style.width = "32px";
                button.style.height = "32px";
                button.style.cursor = "pointer";
                button.style.border = "1px solid #ccc";
                button.style.borderRadius = "4px";
                button.style.backgroundColor = "white";
                button.style.display = "flex";
                button.style.alignItems = "center";
                button.style.justifyContent = "center";
                button.style.boxShadow = "0 1px 3px rgba(0,0,0,0.2)";
                
                const panel = document.createElement("div");
                panel.style.display = "none";
                panel.style.backgroundColor = "white";
                panel.style.padding = "10px";
                panel.style.border = "1px solid #ccc";
                panel.style.borderRadius = "5px";
                panel.style.fontFamily = "sans-serif";
                panel.style.fontSize = "12px";
                panel.style.boxShadow = "0 2px 4px rgba(0,0,0,0.2)";
                panel.style.marginTop = "5px";
                panel.style.minWidth = "100px";

                wrapper.addEventListener("mouseenter", () => panel.style.display = "block");
                wrapper.addEventListener("mouseleave", () => panel.style.display = "none");

                if (layerConfig.pos === "top-right") {{
                    wrapper.style.top = "10px"; wrapper.style.right = "10px";
                }} else if (layerConfig.pos === "bottom-right") {{
                    wrapper.style.bottom = "10px"; wrapper.style.right = "10px";
                }} else if (layerConfig.pos === "bottom-left") {{
                    wrapper.style.bottom = "10px"; wrapper.style.left = "10px";
                }} else {{
                    wrapper.style.top = `${{layerConfig.y}}px`;
                    wrapper.style.left = `${{layerConfig.x}}px`;
                }}
                
                const title = document.createElement("div");
                title.innerText = layerConfig.title;
                title.style.fontWeight = "bold";
                title.style.marginBottom = "8px";
                title.style.borderBottom = "1px solid #eee";
                title.style.paddingBottom = "5px";
                panel.appendChild(title);

                let layers = layerConfig.layers;
                if (!layers) {{
                    layers = Array.from(svg.selectAll("g[id]").nodes()).map(n => n.id);
                }}

                let count = 0;
                layers.forEach(layerId => {{
                    const layer = svg.select("#" + layerId);
                    if (!layer.empty()) {{
                        count++;
                        const row = document.createElement("div");
                        row.style.marginBottom = "5px";
                        row.style.display = "flex";
                        row.style.alignItems = "center";
                        
                        const checkbox = document.createElement("input");
                        checkbox.type = "checkbox";
                        checkbox.id = "chk_" + layerId;
                        checkbox.checked = true;
                        checkbox.style.marginRight = "8px";
                        checkbox.style.cursor = "pointer";
                        
                        checkbox.addEventListener("change", (e) => {{
                            const display = e.target.checked ? "inline" : "none";
                            layer.style("display", display);
                            const legLayer = svg.select("#leg_" + layerId);
                            if (!legLayer.empty()) legLayer.style("display", display);
                        }});
                        
                        const label = document.createElement("label");
                        label.htmlFor = "chk_" + layerId;
                        label.innerText = layerId;
                        label.style.cursor = "pointer";
                        
                        row.appendChild(checkbox);
                        row.appendChild(label);
                        panel.appendChild(row);
                    }}
                }});
                
                if (count > 0) {{
                    wrapper.appendChild(button);
                    wrapper.appendChild(panel);
                    document.getElementById("geoviz-container").appendChild(wrapper);
                }}
            }}
            setTimeout(createLayerControl, 100);
        """

    def _get_export_control_js(self):
        if not self.export_control_config:
            return ""
        ex_config = self.export_control_config
        return f"""
            const exportConfig = {{
                pos: "{ex_config.get('pos')}",
                x: {ex_config.get('x', 10)},
                y: {ex_config.get('y', 50)},
                title: "{ex_config.get('title', 'Export')}"
            }};

            function createExportControl() {{
                const wrapper = document.createElement("div");
                wrapper.style.position = "absolute";
                wrapper.style.zIndex = "1000";

                const button = document.createElement("div");
                button.innerHTML = `<svg width="24" height="24" viewBox="0 0 24 24" fill="#333"><path d="M19 9h-4V3H9v6H5l7 7 7-7zM5 18v2h14v-2H5z"/></svg>`;
                button.style.width = "32px";
                button.style.height = "32px";
                button.style.cursor = "pointer";
                button.style.border = "1px solid #ccc";
                button.style.borderRadius = "4px";
                button.style.backgroundColor = "white";
                button.style.display = "flex";
                button.style.alignItems = "center";
                button.style.justifyContent = "center";
                button.style.boxShadow = "0 1px 3px rgba(0,0,0,0.2)";

                const panel = document.createElement("div");
                panel.style.display = "none";
                panel.style.backgroundColor = "white";
                panel.style.padding = "5px";
                panel.style.border = "1px solid #ccc";
                panel.style.borderRadius = "5px";
                panel.style.marginTop = "5px";
                panel.style.boxShadow = "0 2px 4px rgba(0,0,0,0.2)";
                panel.style.display = "none";
                panel.style.flexDirection = "column";
                panel.style.gap = "5px";

                wrapper.addEventListener("mouseenter", () => panel.style.display = "flex");
                wrapper.addEventListener("mouseleave", () => panel.style.display = "none");

                if (exportConfig.pos === "top-right") {{
                    wrapper.style.top = "50px"; wrapper.style.right = "10px";
                }} else if (exportConfig.pos === "bottom-right") {{
                    wrapper.style.bottom = "50px"; wrapper.style.right = "10px";
                }} else {{
                    wrapper.style.top = `${{exportConfig.y}}px`;
                    wrapper.style.left = `${{exportConfig.x}}px`;
                }}

                const btnSVG = document.createElement("button");
                btnSVG.innerText = "SVG";
                btnSVG.style.cursor = "pointer";
                btnSVG.style.padding = "5px 10px";
                btnSVG.onclick = () => {{
                    geoviz.exportSVG(svg, {{filename: "map.svg"}});
                }};

                const btnPNG = document.createElement("button");
                btnPNG.innerText = "PNG";
                btnPNG.style.cursor = "pointer";
                btnPNG.style.padding = "5px 10px";
                btnPNG.onclick = () => {{
                    geoviz.exportPNG(svg, {{filename: "map.png"}});
                }};

                panel.appendChild(btnSVG);
                panel.appendChild(btnPNG);
                wrapper.appendChild(button);
                wrapper.appendChild(panel);
                document.getElementById("geoviz-container").appendChild(wrapper);
            }}
            setTimeout(createExportControl, 100);
        """

    def _get_opacity_control_js(self):
        if not hasattr(self, "opacity_control_configs") or not self.opacity_control_configs:
            return ""
        
        js_code = ""
        for config in self.opacity_control_configs:
            config_json = json.dumps(config)
            js_code += f"""
            (function() {{
                const opacityConfig = {config_json};
                
                function createOpacityControl() {{
                    const wrapper = document.createElement("div");
                    wrapper.style.position = "absolute";
                    wrapper.style.zIndex = "1000";
                    
                    const button = document.createElement("div");
                    button.innerHTML = `<svg width="24" height="24" viewBox="0 0 24 24" fill="#333"><path d="M17.66 8L12 2.35 6.34 8C4.78 9.56 4 11.64 4 13.64s.78 4.11 2.34 5.66 3.61 2.35 5.66 2.35 4.1-.79 5.66-2.35S20 15.64 20 13.64 19.22 9.56 17.66 8zM6 14c.01-2 .62-3.27 1.76-4.4L12 5.27l4.24 4.38C17.38 10.77 17.99 12 18 14H6z"/></svg>`;
                    button.style.width = "32px";
                    button.style.height = "32px";
                    button.style.cursor = "pointer";
                    button.style.border = "1px solid #ccc";
                    button.style.borderRadius = "4px";
                    button.style.backgroundColor = "white";
                    button.style.display = "flex";
                    button.style.alignItems = "center";
                    button.style.justifyContent = "center";
                    button.style.boxShadow = "0 1px 3px rgba(0,0,0,0.2)";
                    
                    const panel = document.createElement("div");
                    panel.style.display = "none";
                    panel.style.backgroundColor = "white";
                    panel.style.padding = "10px";
                    panel.style.border = "1px solid #ccc";
                    panel.style.borderRadius = "5px";
                    panel.style.fontFamily = "sans-serif";
                    panel.style.fontSize = "12px";
                    panel.style.boxShadow = "0 2px 4px rgba(0,0,0,0.2)";
                    panel.style.marginTop = "5px";
                    panel.style.minWidth = "150px";

                    wrapper.addEventListener("mouseenter", () => panel.style.display = "block");
                    wrapper.addEventListener("mouseleave", () => panel.style.display = "none");

                    if (opacityConfig.pos === "top-right") {{
                        wrapper.style.top = "90px"; wrapper.style.right = "10px";
                    }} else if (opacityConfig.pos === "bottom-right") {{
                        wrapper.style.bottom = "90px"; wrapper.style.right = "10px";
                    }} else {{
                        wrapper.style.top = `${{opacityConfig.y}}px`;
                        wrapper.style.left = `${{opacityConfig.x}}px`;
                    }}
                    
                    const title = document.createElement("div");
                    title.innerText = opacityConfig.title;
                    title.style.fontWeight = "bold";
                    title.style.marginBottom = "8px";
                    title.style.borderBottom = "1px solid #eee";
                    title.style.paddingBottom = "5px";
                    panel.appendChild(title);

                    opacityConfig.layers.forEach(layerId => {{
                        const row = document.createElement("div");
                        row.style.marginBottom = "10px";
                        
                        const label = document.createElement("div");
                        label.innerText = layerId;
                        label.style.fontSize = "11px";
                        label.style.marginBottom = "2px";
                        label.style.color = "#555";
                        
                        const sliderContainer = document.createElement("div");
                        sliderContainer.style.display = "flex";
                        sliderContainer.style.alignItems = "center";
                        
                        const slider = document.createElement("input");
                        slider.type = "range";
                        slider.min = "0";
                        slider.max = "1";
                        slider.step = "0.1";
                        slider.value = "1"; // Default value
                        slider.style.width = "100%";
                        slider.style.cursor = "pointer";

                        // Set initial slider value from layer's current opacity
                        const layerNodeForOpacity = svg.select("#" + layerId);
                        if (!layerNodeForOpacity.empty()) {{
                            const firstShape = layerNodeForOpacity.select("path, circle, rect");
                            if (!firstShape.empty()) {{
                                slider.value = firstShape.style("fill-opacity");
                            }}
                        }}
                        
                        slider.addEventListener("input", (e) => {{
                            const val = e.target.value;
                            const layer = svg.select("#" + layerId);
                            if (!layer.empty()) {{
                                layer.selectAll("path, circle, rect").style("fill-opacity", val);
                            }}
                            const legLayer = svg.select("#leg_" + layerId);
                            if (!legLayer.empty()) {{
                                legLayer.selectAll("path, circle, rect").style("fill-opacity", val);
                            }}
                        }});
                        
                        sliderContainer.appendChild(slider);
                        row.appendChild(label);
                        row.appendChild(sliderContainer);
                        panel.appendChild(row);
                    }});
                    
                    wrapper.appendChild(button);
                    wrapper.appendChild(panel);
                    document.getElementById("geoviz-container").appendChild(wrapper);
                }}
                setTimeout(createOpacityControl, 100);
            }})();
            """
        return js_code



# Configuration file for the Sphinx documentation builder.
import os
import sys
sys.path.insert(0, os.path.abspath('../../'))

project = 'geovizpy'
copyright = '2026, fbxyz'
author = 'fbxyz'
release = '0.1.3'

# -- General configuration ---------------------------------------------------
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.napoleon',
    'sphinx.ext.viewcode',
    'sphinx.ext.githubpages',
]

templates_path = ['_templates']
exclude_patterns = []

# -- Options for HTML output -------------------------------------------------
try:
    import sphinx_rtd_theme
    html_theme = 'sphinx_rtd_theme'
except ImportError:
    html_theme = 'alabaster'

html_static_path = ['_static']

# -- Options for autodoc -----------------------------------------------------
autodoc_member_order = 'bysource'



"""Example script for a choropleth map."""

import json
import os
from geovizpy import Geoviz

# Define output directory
output_dir = os.path.join(os.path.dirname(__file__), "html")
os.makedirs(output_dir, exist_ok=True)

data_path = os.path.join(os.path.dirname(__file__), "data", "world.json")
try:
    with open(data_path) as f:
        world_data = json.load(f)
except FileNotFoundError:
    print(f"{data_path} not found.")
    world_data = {}

viz = Geoviz(projection="EqualEarth")
viz.outline()
viz.graticule(stroke="white", step=30, strokeWidth=1.2)

viz.choro(
    data=world_data,
    var="gdppc",
    strokeWidth=0.3,
    tip=True,
    leg_type="horizontal",
    leg_title="GDP per capita",
    leg_subtitle="($ per inh.)",
    leg_note="Source: worldbank, 2020",
    leg_pos=[410, 370],
    leg_values_round=0,
    leg_missing_text="Missing values"
)

viz.header(
    fontSize=30,
    text="A Choropleth World Map",
    fill="#267A8A",
    fontWeight="bold",
    fontFamily="Tangerine"
)

viz.render_html(os.path.join(output_dir, "choropleth.html"))



"""Example script for a simple world map."""

import json
import os
from geovizpy import Geoviz

# Define output directory
output_dir = os.path.join(os.path.dirname(__file__), "html")
os.makedirs(output_dir, exist_ok=True)

# Load geojson
# Assuming running from repo root or examples folder
data_path = os.path.join(os.path.dirname(__file__), "data", "world.json")
try:
    with open(data_path) as f:
        world_data = json.load(f)
except FileNotFoundError:
    print(f"{data_path} not found.")
    world_data = {}

viz = Geoviz(projection="EqualEarth")
viz.outline(fill="#267A8A")
viz.graticule(stroke="white", strokeWidth=0.4)
viz.path(
    data=world_data,
    fill="#F8D993",
    stroke="#ada9a6",
    strokeWidth=0.5,
    tip="$NAMEen"
)
viz.header(
    fontSize=30,
    text="A Simple World Map",
    fill="#267A8A",
    fontWeight="bold",
    fontFamily="Tangerine"
)
viz.render_html(os.path.join(output_dir, "simple.html"))



"""Example script demonstrating how to use tile layers."""

import json
import os
from geovizpy import Geoviz

# Define output directory
output_dir = os.path.join(os.path.dirname(__file__), "html")
os.makedirs(output_dir, exist_ok=True)

data_path = os.path.join(os.path.dirname(__file__), "data", "world.json")
try:
    with open(data_path) as f:
        world_data = json.load(f)
except FileNotFoundError:
    print(f"Data file not found: {data_path}")
    world_data = {}

viz = Geoviz(projection="mercator", zoomable=True)

# Use the keyword for the tile layer, as per geoviz.js documentation
viz.tile(url="worldStreet")

viz.graticule(stroke="white", strokeWidth=0.4)

# This line might fail if world_data is empty
if world_data:
    viz.prop(
        data=world_data,
        var="pop",
        fill="#f07d75",
        tip="$NAMEen",
        leg_title="Population",
        leg_frame=True,
        leg_pos=[30, 30]
    )

viz.text(
    pos=[0,0],
    text="Source: INSEE 2022",
    fontSize=10,
    fill="#666",
    anchor="end"
)


viz.render_html(os.path.join(output_dir, "tiles.html"))



"""Module for visual effects."""

class EffectsMixin:
    """Mixin class for visual effects."""

    def effect_blur(self, **kwargs):
        """
        Apply a blur effect.
        
        Args:
            id (string): ID of the effect.
            stdDeviation (number): Standard deviation of the blur.
        """
        return self._add_command("effect.blur", kwargs)

    def effect_shadow(self, **kwargs):
        """
        Apply a shadow effect.
        
        Args:
            id (string): ID of the effect.
            dx (number): X offset.
            dy (number): Y offset.
            stdDeviation (number): Blur amount.
            opacity (number): Opacity of the shadow.
            color (string): Color of the shadow.
        """
        return self._add_command("effect.shadow", kwargs)

    def effect_radialGradient(self, **kwargs):
        """
        Apply a radial gradient effect.
        
        Args:
            id (string): ID of the effect.
            stops (list): List of stops (e.g., [{"offset": "0%", "color": "white"}, ...]).
        """
        return self._add_command("effect.radialGradient", kwargs)

    def effect_clipPath(self, **kwargs):
        """
        Apply a clip path effect.
        
        Args:
            id (string): ID of the effect.
            datum (object): GeoJSON to use as clip path.
        """
        return self._add_command("effect.clipPath", kwargs)



"""Example script for adding text labels to a map."""

import json
import os
from geovizpy import Geoviz

# Define output directory
output_dir = os.path.join(os.path.dirname(__file__), "html")
os.makedirs(output_dir, exist_ok=True)

data_path = os.path.join(os.path.dirname(__file__), "data", "world.json")
try:
    with open(data_path) as f:
        world_data = json.load(f)
except FileNotFoundError:
    print(f"{data_path} not found.")
    world_data = {}

countries_to_label = ["Brazil", "China", "United States", "France", "South Africa", "Australia"]
labeled_features = [f for f in world_data.get("features", []) if f["properties"].get("NAMEen") in countries_to_label]
labeled_data = {"type": "FeatureCollection", "features": labeled_features}

viz = Geoviz(projection="EqualEarth")
viz.outline()
viz.graticule(stroke="white", strokeWidth=0.4)

viz.path(datum=world_data, fill="#e0e0e0", stroke="white")

viz.text(
    data=labeled_data,
    text="NAMEen",
    fill="#333",
    fontSize=14,
    fontWeight="bold",
    stroke="white",
    strokeWidth=3,
    paintOrder="stroke"
)



viz.header(text="Map with Labels", fontSize=20)
viz.render_html(os.path.join(output_dir, "labels.html"))



"""Example script demonstrating how to draw lines on a map."""

import json
import os
from geovizpy import Geoviz

# Define output directory
output_dir = os.path.join(os.path.dirname(__file__), "html")
os.makedirs(output_dir, exist_ok=True)

data_path = os.path.join(os.path.dirname(__file__), "data", "world.json")
try:
    with open(data_path) as f:
        world_data = json.load(f)
except FileNotFoundError:
    print(f"{data_path} not found.")
    world_data = {}

lines_data = {
    "type": "FeatureCollection",
    "features": [
        {
            "type": "Feature",
            "properties": {"destination": "New York", "type": "Long-haul"},
            "geometry": {
                "type": "LineString",
                "coordinates": [[2.35, 48.85], [-74.00, 40.71]]
            }
        },
        {
            "type": "Feature",
            "properties": {"destination": "Beijing", "type": "Long-haul"},
            "geometry": {
                "type": "LineString",
                "coordinates": [[2.35, 48.85], [116.40, 39.90]]
            }
        },
        {
            "type": "Feature",
            "properties": {"destination": "Cape Town", "type": "Long-haul"},
            "geometry": {
                "type": "LineString",
                "coordinates": [[2.35, 48.85], [18.42, -33.92]]
            }
        },
        {
            "type": "Feature",
            "properties": {"destination": "Moscow", "type": "Medium-haul"},
            "geometry": {
                "type": "LineString",
                "coordinates": [[2.35, 48.85], [37.61, 55.75]]
            }
        }
    ]
}

viz = Geoviz(projection="EqualEarth")
viz.outline()
viz.graticule(stroke="white", strokeWidth=0.4)

viz.path(datum=world_data, fill="#e0e0e0", stroke="white")

viz.typo(
    data=lines_data,
    var="destination",
    colors="Set1",
    strokeWidth=2,
    fill="none",
    leg_title="Routes from Paris",
    leg_pos=[10, 250]
)

viz.header(text="Map with Lines (Routes)", fontSize=20)
viz.render_html(os.path.join(output_dir, "lines.html"))
